import com.google.soap.search.*;
import java.io.*;
import java.net.*;

public class MostCommonWords {    
	static private final String googleKey = //TODO: Change this to use your own google key
	                                        "f5l9Xf1QFHLiuDQgysqEQSnzXIhQP5Rt";	                                       
	static private final int maxNumberOfResults = 10;
	// Cannot receive more than 10 results per query from Google!
	static private final int maxNumberOfWordsToDisplay = 25;
	static private final int minWordLength = 5;
	// exclude words less than 5 letters long

	static private boolean isBadCharacter(char c) {
		// EFFECTS: Returns true iff c is a character than shouldn't appear in words we care about.
		return (
			c == '<'
				|| c == '>'
				|| c == '|'
				|| c == '/'
				|| c == '='
				|| c == '{'
				|| c == '}'
				|| c == '&'
				|| c == '-');
	}

	static private boolean isSeparator(char c) {
		// EFFECTS: Returns true iff c is a word-separating character.
		return (
			c == '\n'
				|| c == '\t'
				|| c == '\r'
				|| c == ' '
				|| c == '.'
				|| c == ','
				|| c == ';'
				|| c == '('
				|| c == ')');
	}

	public static void main(/*@non_null@*/
	String[] args) {
		if (args.length != 1) {
			System.err.println(
				"Usage: java MostCommonWords <google query string>");
			System.exit(1);
			return; // never reached (but helps ESC/Java)
		}

		String query = args[0];
		WordTally wordTally = new WordTally();

		GoogleSearch search = new GoogleSearch();
		search.setKey(googleKey);
		search.setQueryString(query);
		search.setSafeSearch(true);
		search.setMaxResults(maxNumberOfResults);

		try {
			GoogleSearchResult result = search.doSearch();

			if (result == null) {
				System.err.println("Null search result");
			} else {
				GoogleSearchResultElement[] resultElements =
					result.getResultElements();

				if (resultElements == null) {
					System.err.println("Error: null result elements");
					System.exit(1);
					return; // never reached (but helps ESC/Java)
				}

				for (int i = 0; i < resultElements.length; i++) {
					GoogleSearchResultElement sre = resultElements[i];
					int numwords = 0;

					if (sre == null) {
						// null search element (should never happen?)
					} else {
						System.out.println("Scanning page: " + sre.getURL());

						try {
							URL url = new URL(sre.getURL());
							URLConnection urlConn = url.openConnection();

							if (urlConn != null) {
								InputStream input = urlConn.getInputStream();
								BufferedReader br =
									new BufferedReader(
										new InputStreamReader(input));

								int character;
								StringBuffer word = new StringBuffer();
								character = br.read();
								boolean goodword = true;
								boolean intag = false;

								// Read returns -1 if the end of stream is reached
								while (character != -1) {
									char c = (char) character;

									if (isSeparator(c)) {
										if (goodword
											&& !intag
											&& word.length() >= minWordLength) {
											wordTally.tallyWord(
												word.toString());
											numwords++;
										}
										word = new StringBuffer();
										goodword = true;
									} else {
										// Only count "interesting" words
										if (c == '<') {
											intag = true;
										}
										if (intag && c == '>') {
											intag = false;
										}

										if (goodword) {
											if (isBadCharacter(c)) {
												goodword = false;
											} else {
												word = word.append(c);
											}
										}
									}
									character = br.read();
								}
								System.out.println(
									"   Good words: " + numwords);
								numwords = 0;
							}
						} catch (Exception e) {
							System.err.println("Error reading: " + e);
						}
					}
				}
			}

			System.out.println("Different words: " + wordTally.numberOfWords());
			for (int i = 1;
				i <= maxNumberOfWordsToDisplay && i < wordTally.numberOfWords();
				i++) {
				String word = wordTally.getRankedWord(i);

				System.out.println(
					i + ". " + word + " (" + wordTally.getTally(word) + ")");
			}
		} catch (Exception e) {
			System.err.println(
				"An error occured during a search or openning a connection to a Url");
		}
		System.out.println("Finished processing results successfully.");
	}
}
